import { Component, OnInit } from "@angular/core";
import { CapabilityRequest } from "../../model/capability-request";
import { ActivatedRoute } from "@angular/router";
import { DataRetrieverService } from "../../services/data-retriever.service";
import { CapabilityExecution } from "../../model/capability-execution";
import { Capability } from "../../model/capability";
import { CapabilityRequestService } from "../../services/capability-request.service";
import { CapabilityVersion } from "../../model/capability-version";

@Component({
  selector: "app-capability-request",
  templateUrl: "./capability-request.component.html",
  styleUrls: ["./capability-request.component.scss"],
})
export class CapabilityRequestComponent implements OnInit {
  public capabilityRequest: CapabilityRequest;
  public capability: Capability;
  public capabilityExecution: CapabilityExecution;
  public currentVersion: CapabilityVersion;

  // Observer for capability request objects
  private capabilityRequestObserver = {
    next: (request) => {
      this.capabilityRequest = request;
      this.capabilityExecution = request.current_execution;
      this.dataRetriever
        .getCapability(this.capabilityRequest.capability_name)
        .subscribe(this.capabilityObserver);
    },
    error: (error) => console.error("Error when retrieving capability request:" + error),
  };
  // Observer for capability objects
  private capabilityObserver = {
    next: (capability) => {
      this.capability = capability;
    },
    error: (error) => console.error("Error when retrieving capability:" + error),
  };
  // Observer for capability version objects
  private capabilityVersionObserver = {
    next: (capabilityVersion) => {
      if (capabilityVersion !== undefined) {
        this.currentVersion = capabilityVersion;
      } else {
        console.error("Current version returned undefined.");
      }
    },
    error: (error) => console.error("Error when retrieving current version:" + error),
  };

  constructor(
    private route: ActivatedRoute,
    private dataRetriever: DataRetrieverService,
    private capabilityRequestService: CapabilityRequestService,
  ) {
    const requestID = parseInt(this.route.snapshot.paramMap.get("id"));
    this.capabilityRequestService
      .getCapabilityRequest(requestID)
      .subscribe(this.capabilityRequestObserver);
    this.capabilityRequestService
      .getLatestVersion(requestID)
      .subscribe(this.capabilityVersionObserver);
  }

  ngOnInit(): void {}
}
